@@ -90,10 +90,16 @@ sub print_rec_time {
 }
 
 sub open_cleanup {
-  my ($class) = @_;
+  my ($class, $socket) = @_;
+
+  $socket = "/var/spool/postfix/public/cleanup"
+    unless defined $socket;
+
   my $self = IO::Socket::UNIX->new(Type => SOCK_STREAM,
-  				   Peer => "/var/spool/postfix/public/cleanup");
-  die qq[Couldn't open unix socket "/var/spool/postfix/public/cleanup": $!] unless ref $self;
+  				   Peer => $socket);
+  die qq(Couldn't open unix socket "$socket": $!) unless ref $self;
+  # allow buffered writes
+  $self->autoflush(0);
   bless ($self, $class);
   $self->init();
   return $self;
@@ -157,7 +163,7 @@ $transaction is supposed to be a Qpsmtpd::Transaction object.
 sub inject_mail {
   my ($class, $transaction) = @_;
 
-  my $strm = $class->open_cleanup();
+  my $strm = $class->open_cleanup($transaction->notes('postfix-queue-socket'));
 
   my %at = $strm->get_attr;
   my $qid = $at{queue_id};
@@ -170,7 +176,7 @@ sub inject_mail {
   }
   # add an empty message length record.
   # cleanup is supposed to understand that.
-  # see src/pickup/pickup.c 
+  # see src/pickup/pickup.c
   $strm->print_rec('REC_TYPE_MESG', "");
 
   # a received header has already been added in SMTP.pm
@@ -197,6 +203,6 @@ sub inject_mail {
   $strm->close();
   return wantarray ? ($status, $qid, $reason || "") : $status;
 }
-  
+
 1;
 # vim:sw=2
